data = (
'sswals',    # 0x00
'sswalt',    # 0x01
'sswalp',    # 0x02
'sswalh',    # 0x03
'sswam',    # 0x04
'sswab',    # 0x05
'sswabs',    # 0x06
'sswas',    # 0x07
'sswass',    # 0x08
'sswang',    # 0x09
'sswaj',    # 0x0a
'sswac',    # 0x0b
'sswak',    # 0x0c
'sswat',    # 0x0d
'sswap',    # 0x0e
'sswah',    # 0x0f
'sswae',    # 0x10
'sswaeg',    # 0x11
'sswaegg',    # 0x12
'sswaegs',    # 0x13
'sswaen',    # 0x14
'sswaenj',    # 0x15
'sswaenh',    # 0x16
'sswaed',    # 0x17
'sswael',    # 0x18
'sswaelg',    # 0x19
'sswaelm',    # 0x1a
'sswaelb',    # 0x1b
'sswaels',    # 0x1c
'sswaelt',    # 0x1d
'sswaelp',    # 0x1e
'sswaelh',    # 0x1f
'sswaem',    # 0x20
'sswaeb',    # 0x21
'sswaebs',    # 0x22
'sswaes',    # 0x23
'sswaess',    # 0x24
'sswaeng',    # 0x25
'sswaej',    # 0x26
'sswaec',    # 0x27
'sswaek',    # 0x28
'sswaet',    # 0x29
'sswaep',    # 0x2a
'sswaeh',    # 0x2b
'ssoe',    # 0x2c
'ssoeg',    # 0x2d
'ssoegg',    # 0x2e
'ssoegs',    # 0x2f
'ssoen',    # 0x30
'ssoenj',    # 0x31
'ssoenh',    # 0x32
'ssoed',    # 0x33
'ssoel',    # 0x34
'ssoelg',    # 0x35
'ssoelm',    # 0x36
'ssoelb',    # 0x37
'ssoels',    # 0x38
'ssoelt',    # 0x39
'ssoelp',    # 0x3a
'ssoelh',    # 0x3b
'ssoem',    # 0x3c
'ssoeb',    # 0x3d
'ssoebs',    # 0x3e
'ssoes',    # 0x3f
'ssoess',    # 0x40
'ssoeng',    # 0x41
'ssoej',    # 0x42
'ssoec',    # 0x43
'ssoek',    # 0x44
'ssoet',    # 0x45
'ssoep',    # 0x46
'ssoeh',    # 0x47
'ssyo',    # 0x48
'ssyog',    # 0x49
'ssyogg',    # 0x4a
'ssyogs',    # 0x4b
'ssyon',    # 0x4c
'ssyonj',    # 0x4d
'ssyonh',    # 0x4e
'ssyod',    # 0x4f
'ssyol',    # 0x50
'ssyolg',    # 0x51
'ssyolm',    # 0x52
'ssyolb',    # 0x53
'ssyols',    # 0x54
'ssyolt',    # 0x55
'ssyolp',    # 0x56
'ssyolh',    # 0x57
'ssyom',    # 0x58
'ssyob',    # 0x59
'ssyobs',    # 0x5a
'ssyos',    # 0x5b
'ssyoss',    # 0x5c
'ssyong',    # 0x5d
'ssyoj',    # 0x5e
'ssyoc',    # 0x5f
'ssyok',    # 0x60
'ssyot',    # 0x61
'ssyop',    # 0x62
'ssyoh',    # 0x63
'ssu',    # 0x64
'ssug',    # 0x65
'ssugg',    # 0x66
'ssugs',    # 0x67
'ssun',    # 0x68
'ssunj',    # 0x69
'ssunh',    # 0x6a
'ssud',    # 0x6b
'ssul',    # 0x6c
'ssulg',    # 0x6d
'ssulm',    # 0x6e
'ssulb',    # 0x6f
'ssuls',    # 0x70
'ssult',    # 0x71
'ssulp',    # 0x72
'ssulh',    # 0x73
'ssum',    # 0x74
'ssub',    # 0x75
'ssubs',    # 0x76
'ssus',    # 0x77
'ssuss',    # 0x78
'ssung',    # 0x79
'ssuj',    # 0x7a
'ssuc',    # 0x7b
'ssuk',    # 0x7c
'ssut',    # 0x7d
'ssup',    # 0x7e
'ssuh',    # 0x7f
'ssweo',    # 0x80
'ssweog',    # 0x81
'ssweogg',    # 0x82
'ssweogs',    # 0x83
'ssweon',    # 0x84
'ssweonj',    # 0x85
'ssweonh',    # 0x86
'ssweod',    # 0x87
'ssweol',    # 0x88
'ssweolg',    # 0x89
'ssweolm',    # 0x8a
'ssweolb',    # 0x8b
'ssweols',    # 0x8c
'ssweolt',    # 0x8d
'ssweolp',    # 0x8e
'ssweolh',    # 0x8f
'ssweom',    # 0x90
'ssweob',    # 0x91
'ssweobs',    # 0x92
'ssweos',    # 0x93
'ssweoss',    # 0x94
'ssweong',    # 0x95
'ssweoj',    # 0x96
'ssweoc',    # 0x97
'ssweok',    # 0x98
'ssweot',    # 0x99
'ssweop',    # 0x9a
'ssweoh',    # 0x9b
'sswe',    # 0x9c
'ssweg',    # 0x9d
'sswegg',    # 0x9e
'sswegs',    # 0x9f
'sswen',    # 0xa0
'sswenj',    # 0xa1
'sswenh',    # 0xa2
'sswed',    # 0xa3
'sswel',    # 0xa4
'sswelg',    # 0xa5
'sswelm',    # 0xa6
'sswelb',    # 0xa7
'sswels',    # 0xa8
'sswelt',    # 0xa9
'sswelp',    # 0xaa
'sswelh',    # 0xab
'sswem',    # 0xac
'ssweb',    # 0xad
'sswebs',    # 0xae
'sswes',    # 0xaf
'sswess',    # 0xb0
'ssweng',    # 0xb1
'sswej',    # 0xb2
'sswec',    # 0xb3
'sswek',    # 0xb4
'sswet',    # 0xb5
'sswep',    # 0xb6
'ssweh',    # 0xb7
'sswi',    # 0xb8
'sswig',    # 0xb9
'sswigg',    # 0xba
'sswigs',    # 0xbb
'sswin',    # 0xbc
'sswinj',    # 0xbd
'sswinh',    # 0xbe
'sswid',    # 0xbf
'sswil',    # 0xc0
'sswilg',    # 0xc1
'sswilm',    # 0xc2
'sswilb',    # 0xc3
'sswils',    # 0xc4
'sswilt',    # 0xc5
'sswilp',    # 0xc6
'sswilh',    # 0xc7
'sswim',    # 0xc8
'sswib',    # 0xc9
'sswibs',    # 0xca
'sswis',    # 0xcb
'sswiss',    # 0xcc
'sswing',    # 0xcd
'sswij',    # 0xce
'sswic',    # 0xcf
'sswik',    # 0xd0
'sswit',    # 0xd1
'sswip',    # 0xd2
'sswih',    # 0xd3
'ssyu',    # 0xd4
'ssyug',    # 0xd5
'ssyugg',    # 0xd6
'ssyugs',    # 0xd7
'ssyun',    # 0xd8
'ssyunj',    # 0xd9
'ssyunh',    # 0xda
'ssyud',    # 0xdb
'ssyul',    # 0xdc
'ssyulg',    # 0xdd
'ssyulm',    # 0xde
'ssyulb',    # 0xdf
'ssyuls',    # 0xe0
'ssyult',    # 0xe1
'ssyulp',    # 0xe2
'ssyulh',    # 0xe3
'ssyum',    # 0xe4
'ssyub',    # 0xe5
'ssyubs',    # 0xe6
'ssyus',    # 0xe7
'ssyuss',    # 0xe8
'ssyung',    # 0xe9
'ssyuj',    # 0xea
'ssyuc',    # 0xeb
'ssyuk',    # 0xec
'ssyut',    # 0xed
'ssyup',    # 0xee
'ssyuh',    # 0xef
'sseu',    # 0xf0
'sseug',    # 0xf1
'sseugg',    # 0xf2
'sseugs',    # 0xf3
'sseun',    # 0xf4
'sseunj',    # 0xf5
'sseunh',    # 0xf6
'sseud',    # 0xf7
'sseul',    # 0xf8
'sseulg',    # 0xf9
'sseulm',    # 0xfa
'sseulb',    # 0xfb
'sseuls',    # 0xfc
'sseult',    # 0xfd
'sseulp',    # 0xfe
'sseulh',    # 0xff
)
