from messagebus import TokenPriority, TokenCreator


class LoyaltyToken(object):
    MESSAGE_GROUP = "111"

    @staticmethod
    def create_token(
        number,  # type: str
    ):
        # type: (...) -> int

        return TokenCreator.create_token(
            priority=TokenPriority.low,
            message_group=LoyaltyToken.MESSAGE_GROUP,
            number=number,
        )
