from ._LoyaltyToken import LoyaltyToken
from messagebus import TokenCreator, TokenPriority


class LoyaltyResponseToken(LoyaltyToken):
    TK_LOYALTY_ERROR = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group=LoyaltyToken.LOYALTY_MESSAGE_GROUP,
        number="F002"
    )
