from typing import Dict


class Customer(object):
    def __init__(
        self,
        customer,    # type: Dict
    ):
        # type: (...) -> None

        self.id = customer.get("id", None)
        self.name = customer.get("name", None)
        self.document = customer.get("document", None)
