class Action(object):
    def __init__(self, obj):
        self.order = int(obj.get("order")) if obj.get("order") else None
        self.name = obj.get("name")
        self.value = obj.get("value")
        self.times_to_apply = None
        self.seat = obj.get("seat", None)
        self.max_value = obj.get(
            "maxValue",
            None,
        )
        self.tender_id = int(obj["tenderId"]) if "tenderId" in obj and obj["tenderId"] is not None else None
