# encoding: utf-8

from typing import Dict


class JustificationDiscountBodyDto(object):
    def __init__(
        self,
        justifications,     # type: Dict
    ):
        # type: (...) -> None

        self.justifications = justifications
