def build_response(products):
    response_list = {"rupturedItems": []}
    for product_code in products:
        response_list["rupturedItems"].append(int(product_code))
    return response_list


class GetRupturedProductsResponse(object):
    def __init__(self, ruptured_products):
        # type: (list) -> None

        self.ruptured_products = build_response(ruptured_products)
