# encoding: utf-8
from datetime import datetime


class JustificationDiscountHeaderDto(object):

    def __init__(
        self,
        operator,       # type: str
        pos_id,         # type: str
        start_date,     # type: str
        end_date,       # type: str
    ):
        # type: (...) -> None

        self.title = "$JUSTIFICATION_DISCOUNT_REPORT"
        self.pos_id = pos_id
        self.current_date = datetime.now().strftime("%d/%m/%Y %H:%M:%S")
        self.operator = operator if operator is not None else "$ALL_OPERATORS"
        self.start_date = datetime.strptime(start_date, '%Y%m%d').strftime('%d/%m/%y')
        self.end_date = datetime.strptime(end_date, '%Y%m%d').strftime('%d/%m/%y')
